-- Author:     Andy Stewart <lazycat.manatee@gmail.com>
-- Maintainer: Andy Stewart <lazycat.manatee@gmail.com>
-- 
-- Copyright (C) 2010 ~ 2011 Andy Stewart, all rights reserved.
-- 
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- any later version.
-- 
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.

{-# LANGUAGE DeriveDataTypeable #-}
{-# LANGUAGE TemplateHaskell #-}
module Manatee.Extension.Terminal.TerminalBuffer where

import Control.Applicative
import Control.Concurrent.STM 
import DBus.Client hiding (Signal)
import Data.Binary
import Data.DeriveTH
import Data.Typeable
import Manatee.Core.Config
import Manatee.Core.Types
import Manatee.Extension.Terminal.PageMode
import Manatee.Toolkit.General.STM

data TerminalBuffer = 
    TerminalBuffer {terminalBufferDirectory             :: TVar String
                   ,terminalBufferClient                :: Client
                   ,terminalBufferPageId                :: PageId
                   ,terminalBufferMode                  :: PageMode
                   ,terminalBufferState                 :: TVar TerminalState
                   }
    deriving Typeable

data TerminalState =
    TerminalState {terminalStateScrolledPosition  :: (Double, Double)}

-- | Init state.
terminalInitState :: TerminalState
terminalInitState =
    TerminalState (0, 0)

-- | New terminal buffer.
terminalBufferNew :: String -> [String] -> Client -> PageId -> CustomizeWrap -> IO TerminalBuffer
terminalBufferNew dir _ client pageId _ = 
  TerminalBuffer <$> newTVarIO dir
                 <*> pure client
                 <*> pure pageId
                 <*> pure terminalMode
                 <*> newTVarIO terminalInitState
                 
-- | Write state.
terminalBufferWriteState :: TerminalBuffer -> FilePath -> IO ()
terminalBufferWriteState buffer path = do
  state <- readTVarIO $ terminalBufferState buffer
  writeConfigPath path state

-- | Read state.
terminalBufferReadState :: TerminalBuffer -> FilePath -> IO ()  
terminalBufferReadState buffer path = do
  state <- readConfigPath path terminalInitState
  writeTVarIO (terminalBufferState buffer) state
  
$(derive makeBinary ''TerminalState)

